import Ember from 'ember';
const { Service, isEmpty, run, Object: EmObject, A: EmArray } = Ember;

export default Service.extend({
  init() {
    this._super();
    this.set('actives', EmObject.create());
    this._alive = {};
    this._counter = 1;
  },
  show(sourceId, name, component, order) {
    order = order > 0 ? order : this._counter++;
    this._alive[sourceId] = {
      target: name || 'default',
      component,
      order: order
    };
    this._schedule();
  },
  clear(sourceId) {
    delete this._alive[sourceId];
    this._schedule();
  },
  _schedule() {
    run.scheduleOnce('afterRender', this, this._process);
  },
  _process() {
    let newActives = {};
    let alive = this._alive;

    Object.keys(alive).forEach(sourceId => {
      let newActive = {};
      let { target, component, order } = alive[sourceId];
      let existing = newActives[target];
      if (isEmpty(existing) || isEmpty(existing[sourceId])) {
        newActives[target] = newActives[target] || EmArray();

        newActive = component ? { component, order } : null;

        let idx = order - 1;
        if (idx <= newActives[target].length) {
          let source = newActives[target].objectAt(idx);
          if (!isEmpty(source) && (source.get('order') === order)) {
            newActives[target].removeAt(idx);
          }
          newActives[target].insertAt(idx, EmObject.create(newActive));
        } else {
          newActives[target].addObject(EmObject.create(newActive));
        }
      }
    });
    this.set('actives', EmObject.create(newActives));
  }
});
